C==================================================================
C
C     CONTOUR MAP DRAWING PROGRAM MECO for MaxEnt- and Fourier Program
C
C     ORIGINAL PROGRAMMED                BY M.SATO      88-02
C     ARRANGED FOR MEMTARO DATA          BY S.KUMAZAWA  89-08-11
C     MS-DOS VERSION                     BY S.KUMAZAWA  89-08-24
C     ADD 2 X 2 UNIT CELL MODE           BY S.KUMAZAWA  90-04-17
C     ADD PROJECTION & ORIGIN SHIFT MODE BY Y.KUBOTA    91-04-11
C     MODIFIED PROGRAM FOR MEED DATA     BY S.KUMAZAWA  91-10-31
C     BUG FIXED FOR TRANSVERSE ROUTINE   BY S.KUMAZAWA  91-11-08
C     BUG FIXED FOR AXIS NAMING          BY S.KUMAZAWA  91-11-15
C     Many things changed                   K. Burger   1995, April 1996
C     Arbitrary sections interpolation added   k.b.     Oct. 1996
C
C====================================================================
C     - Output routine written for SURFER graphics program on PC/WINDOWS
C       K. Burger and Dr. P. Schiebel, mai 1994.
C     - Incorporated unit cells with angles not equal 90 degrees.
C       the triangles at two edges missing to a rectangular map are filled
C       K. Burger 1995
C     - "bugs" removed; in density file free format ix,iy,RDEN
C       Jan. 1996 kb
C     - TITLE with 200 char's, march 1996 k.b.
C     - Input Format of density-data changed 
C       Program modularised:  April 1996
C     - interpolation routine mapxxx added for arbitrary sections.
C       October 1996, kb.
C====================================================================
      PROGRAM mecox
C
      IMPLICIT none
      integer*4 nasym,masym,ia
C
C     Parameters: 
C       masym = max. number of pixels in asymmetric unit of cell
C       mpix = max. number of pixels along one cell axis (resolution)
C
      integer mpix
      real*8 vpx1,vpy1,vpx2,vpy2,wpx1,wpy1,wpx2,wpy2
      PARAMETER (masym=135000,mpix=102)
      PARAMETER (vpx1=0.0,vpy1=0.0,vpx2=1.6,vpy2=1.0)
      PARAMETER (wpx1=vpx1-0.05,wpy1=vpy1-0.2)
      PARAMETER (wpx2=vpx2-0.05,wpy2=vpy2-0.2)
C       DELC: reconstructed density in whole cell
C       CELL: cell parameters a,b,c,alpha,beta,gamma
C       NPIX: no. of pixels in cell-axes directions
C       RDEN: array with density in asymmetric unit
C       IASYM: corresponding ix,iy,iz coordinates of pixels in a.u.
C       IROT,RTRS,RSUB: symmetry operators
C       imapXX: with XX=00,11,so,sx,sy,uc,tr,sp,se,sf,st,px,py
C               are input variables of the subprograms MAP110,MAP100 and 
C               MAPxxx. Also maplx,maply. They save the last input
C               choices of a user.
C
C     *** variables connected with the scattering-density values in the 
C     asymmetric unit of the cell and in the whole reconstructed cell:
      real*8 RDEN(MASYM), DELC(0:mpix,0:mpix,0:mpix)
C
C     *** coordinates of the pixels:
      integer NPIX(3), IASYM(3,MASYM)
C
C     *** 2-dim map: projection of section.
      real*8 D(2*mpix+1,2*mpix+1),vx,vy,P1(3),P2(3),P3(3)
      integer lx,ly,nmx,nmy
C
C     *** cell-constants and others (from PRECAL subroutine):
      real*8 CELL(6),rh,rk,rl,as,bs,cs,ga,gb,gg,av,sv,dv,dg,db,da,sg,
     $     sb,sa,cg,cb,ca,rdint,ff1,ff2,pi,rad,theta,dmin,dmax
C
C     *** variables connected with symmetry operators:
      integer IROT(3,3,48),nsym,ncen,nsub
      real*8 RTRS(3,48),RSUB(3,4)
C
C     *** other variables:
      CHARACTER title*200,filden*50
      CHARACTER axisX*5,axisY*5
      integer imap00,imap11,imapor,imapso,imapsx,imapsy,imapuc,imaptr,
     $        imapsp,imapse,imapsf,imapst,imappx,imappy,i,j,k,nyes
      real*8 rsum,rsumold,rr,rmin,rmax,wert,maplx,maply
      logical CR,error
C
C     *** default values of the input variables ***
      imap00=4
      imap11=0
      imapor=0
      imapso=1
      imapsx=0
      imapsy=0
      imapuc=0
      imaptr=0
      imapsp=0
      imapse=0
      imapsf=0
      imapst=0
      maplx=5.0
      maply=5.0
      imappx=30
      imappy=30
      
 1000 FORMAT((' '))
 1090 format(2(3(I3,1X),F18.6,1x))
 2000 FORMAT(BN,I3)
 2010 FORMAT(A50)
 3010 FORMAT(/' ==============================================='
     $       /'   CONTOUR MAP DRAWING PROGRAM  ( MECO )',
     $       /'   VERSION 3.22  COPYRIGHTS (C) S.KUMAZAWA',
     $       /'   Last modification: K. Burger, in April 1998',
     $       /'   Universitaet Tuebingen, Germany',
     $       /' ===============================================',
     $      //' Input density file''s name   = ',$)
 3020 FORMAT( ' Are you sure ?  YES(0)/NO(1) = ',$)
 3040 FORMAT(' Which section''s map do you want to draw ?'
     $       /' 1:(110), 2:(100), 3:(010), 4:(001), 5:(xxx)',
     $       ' 6:New file, 0:Exit [default=',i1,'] ',$)
C
C
C*** READ DENSITY FILE & RECONSTRUCT DENSITY ***
C
   10 WRITE(*,1000)
      WRITE(*,3010)
      READ(*,2010,ERR=10) filden
      print *
      print *,'Please wait ...'
C
C     *** read crystal parameters, asymmetric unit and symmetry operators
      CALL READD2(filden,masym,nasym,IASYM,RDEN,
     $   title,CELL,mpix,npix,nsym,ncen,nsub,IROT,RTRS,RSUB)

      print *,title
      print *,'First 10 values of density:'
      write(*,1090) ((iasym(j,i),j=1,3),RDEN(i),i=1,10)
      print *

C     *** display of symmetry info ***
C     CALL DSPSYM(nsg,nset,aspg,nsym,ncen,nsub,
C    $  IROT,RTRS,RSUB,CELL)
C
C     *** calculate some constants 
      CALL PRECAL(0,0,0,pi,rad,ca,cb,cg,sa,sb,sg,
     $   da,db,dg,dv,sv,as,bs,cs,ga,gb,gg,rh,rk,rl,av,
     $   rdint,1.d0,ff1,ff2,CELL,NPIX)

      rmax=RDEN(1)
      rmin=rmax
      rsum=0d0
      do 39, ia=1,NASYM
	rr=RDEN(ia)
        if (rr .gt. rmax) rmax=rr
        if (rr .lt. rmin) rmin=rr 
	rsum=rsum+rr
39    continue
      print *,'Asymmetric unit of the crystal cell:'
4099  format(a,f12.4)
4098  format(a,i12)
      write(*,4099) '   max=',rmax
      write(*,4099) '   min=',rmin
      rsum=rsum*FF1
      write(*,4098) '   Points in asym. unit= ',nasym
      write(*,4099) '   Scattering power sum= ', rsum
      print *,'Please wait ...'

C     *** reconstruct the whole-cell density-distribution DELC
      CALL RECNST(mpix,NPIX,masym,nasym,RDEN,IASYM,CELL,
     $   nsym,ncen,nsub,IROT,RTRS,RSUB,DELC)
C
      rsumold=rsum
      rsum=0
      do 40 i=0,NPIX(1)-1
      do 40 j=0,NPIX(2)-1
      do 40 k=0,NPIX(3)-1
	rsum=rsum+DELC(i,j,k)
40    continue
      rsum = rsum*FF1
      write(*,4099) ' Total scattering power=   ', rsum
      write(*,4099) ' Ratio to asym. unit sc.p.=', rsum/rsumold
C
C***  now select sections and make files for the SURFER graphics program ***
C
   50 WRITE(*,1000)
   51 WRITE(*,3040) imap00
      CALL abfrage(wert,i,CR,error)
      if (error) goto 51
      if (.not.CR) then
         if (i.gt.6 .or. i.lt.0) then
           print *,'wrong input'
           goto 51
         else 
          if (i.eq.6) goto 10		!new file
          imap00=i
        endif
      endif

      if (imap00.eq.0) then
C111     write(*,1112)
C1112    format(' Are you sure? (0=yes, 1=no) ',$)
C        read (*,2000,ERR=111) i
        i=0
        if (i.eq.0) then
          write(*,*) 'Finished.'
          STOP
        else 
          goto 51
        endif
      elseif (imap00.eq.1) then
        CALL MAP110(imap00,mpix,NPIX,nmx,nmy,vx,vy,theta,rad,
     $    CELL,DELC,D,lx,ly,dmin,dmax,axisX,axisY,
     $    imap11,imapso,imapsx,imapsy,imapuc,imaptr)
      elseif (imap00.eq.2.or.imap00.eq.3.or.imap00.eq.4) then
        CALL MAP100(imap00,mpix,NPIX,nmx,nmy,vx,vy,theta,rad,
     $    CELL,DELC,D,lx,ly,dmin,dmax,axisX,axisY,
     $    imapso,imapsx,imapsy,imapuc,imaptr,
     $    imapsp,imapse,imapsf,imapst)
      elseif (imap00.eq.5) then
        CALL MAPXXX(mpix,NPIX,filden,title,
     $    CELL,DELC,ca,cb,cg,sa,sb,sg,as,bs,cs,ga,gb,gg,pi,
     $    P1,P2,P3,imapor,maplx,maply,imappx,imappy)
        GOTO 50
      else
	GOTO 50
      endif
C     *** are you sure ? ***
27    WRITE(*,3020)
      READ(*,2000,ERR=27) nyes
      IF (nyes.NE.0) GO TO 50
C
C     *** output of result ***
      CALL MECORES(mpix,rad,theta,vx,vy,dmin,dmax,
     $             imap00,lx,ly,axisx,axisy,D)
      GO TO 50
C
      END
