C====================================================================
C
C     CONTOUR MAP DRAWING PROGRAM MECO for MaxEnt- and Fourier Program
C
C     ORIGINAL PROGRAMMED                BY M.SATO      88-02
C     ARRANGED FOR MEMTARO DATA          BY S.KUMAZAWA  89-08-11
C     MS-DOS VERSION                     BY S.KUMAZAWA  89-08-24
C     ADD 2 X 2 UNIT CELL MODE           BY S.KUMAZAWA  90-04-17
C     ADD PROJECTION & ORIGIN SHIFT MODE BY Y.KUBOTA    91-04-11
C     MODIFIED PROGRAM FOR MEED DATA     BY S.KUMAZAWA  91-10-31
C     BUG FIXED FOR TRANSVERSE ROUTINE   BY S.KUMAZAWA  91-11-08
C     BUG FIXED FOR AXIS NAMING          BY S.KUMAZAWA  91-11-15
C     Many things changed                   K. Burger   1995, April 1996
C
C====================================================================
C     - Ausgaberoutine angepasst fuer SURFER-Graphik:
C       Modifiziert von K. Burger und Dr. P. Schiebel, Mai 1994.
C     - Modifiziert fuer beliebig schiefe Zellen und Ausgabe fuer das
C       Programm SURFER fuer WINDOWS:          K. Burger 1995
C     - "bugs" entfernt; in Dichte-Datei haben ix,iy,iz,RDEN 
C       freies Format: Jan. 1996 kb
C     - Mod. TITLE 200 char's, Maerz 1996 k.b.
C     - Input Format of density-data changed 
C       Program modularised:  April 1996
C====================================================================
      PROGRAM MECO
C
      IMPLICIT INTEGER (I-N)
      IMPLICIT REAL*8 (A-H,O-Z)
      integer*4 nasym,masym,ia
C
C     Parameters: 
C       masym = max. number of pixels in asymmetric unit of cell
C       mpix = max. number of pixels along one cell axis (resolution)
C
      PARAMETER (masym=135000,mpix=102)
      PARAMETER (vpx1=0.0,vpy1=0.0,vpx2=1.6,vpy2=1.0)
      PARAMETER (wpx1=vpx1-0.05,wpy1=vpy1-0.2)
      PARAMETER (wpx2=vpx2-0.05,wpy2=vpy2-0.2)
C       DELC: reconstructed density in whole cell
C       CELL: cell parameters a,b,c,alpha,beta,gamma
C       NPIX: no. of pixels in cell-axes directions
C       RDEN: array with density in asymmetric unit
C       IASYM: corresponding ix,iy,iz coordinates of pixels in a.u.
C       IROT,RTRS,RSUB: symmetry operators
C
C     *** variables connected with the scattering-density values in the 
C     asymmetric unit of the cell and in the whole reconstructed cell:
      dimension RDEN(MASYM),DELC(0:mpix,0:mpix,0:mpix)
C
C     *** variables connected with the coordinates of the pixels:
      integer NPIX(3),IASYM(3,MASYM)
C
C     *** 2-dim map
      dimension D(2*mpix+1,2*mpix+1)
C
C     *** cell-constants and others (from PRECAL subroutine):
      real*8 CELL(6),rh,rk,rl,as,bs,cs,ga,gb,gg,
     $     rdint,ff1,ff2,pi,rad
C
C     *** variables connected with symmetry operators:
      integer IROT(3,3,48),nsym,ncen,nsub
      real*8 RTRS(3,48),RSUB(3,4)
C
C     *** names of x and y axes
      CHARACTER axisX*5,axisY*5
C
C     *** other variables:
      CHARACTER title*200,filden*50
C
 1000 FORMAT((' '))
 1090 format(2(3(I3,1X),F18.6,1x))
 2000 FORMAT(BN,I3)
 2010 FORMAT(A50)
 3010 FORMAT(/' ==============================================='
     $      //'     CONTOUR MAP DRAWING PROGRAM  ( MECO )'
     $      //'     VERSION 3.22  COPYRIGHTS (C) S.KUMAZAWA'
     $      //'     Last modification: K. Burger, april 1996'
     $      //' ==============================================='
     $      //' INPUT DENSITY FILE''S NAME    = ')
 3020 FORMAT( ' ARE YOU SURE ?  YES(0)/NO(1) = ')
 3040 FORMAT(' WHICH SECTION''S MAP DO YOU WANT TO DRAW ?'
     $       /'   1: (110), 2: (100), 3: (010), 4: (001),',
     $       ' 0: EXIT TO SYSTEM   = ')
C
C
C*** READ DENSITY FILE & RECONSTRUCT DENSITY ***
C
   10 WRITE(*,1000)
      WRITE(*,3010)
      READ(*,2010,ERR=10) filden
      WRITE(*,3020)
      READ(*,2000,ERR=10) nyes
      IF(nyes.NE.0) GOTO 10
C
C     *** read crystal parameters, asymmetric unit and symmetry operators
      CALL READD2(filden,masym,nasym,IASYM,RDEN,
     $   title,CELL,mpix,npix,nsym,ncen,nsub,IROT,RTRS,RSUB)

      print *,title
      print *,'First 10 values of density:'
      write(*,1090) ((iasym(j,i),j=1,3),RDEN(i),i=1,10)
      print *

C     *** display of symmetry info ***
C     CALL dspsym(nsg,nset,aspg,nsym,ncen,nsub,
C    $  IROT,RTRS,RSUB,CELL)
C
C     *** calculate some constants 
      CALL PRECAL(0,0,0,pi,rad,ca,cb,cg,sa,sb,sg,
     $   da,db,dg,dv,sv,as,bs,cs,ga,gb,gg,rh,rk,rl,av,
     $   rdint,1.d0,ff1,ff2,CELL,NPIX)

      rmax=RDEN(1)
      rmin=rmax
      rsum=0d0
      do 39, ia=1,NASYM
	rr=RDEN(ia)
        if (rr .gt. rmax) rmax=rr
        if (rr .lt. rmin) rmin=rr 
	rsum=rsum+rr
39    continue
      print *,'Asymmetric unit of the crystal cell:'
4099  format(a,f12.4)
4098  format(a,i12)
      write(*,4099) '   max=',rmax
      write(*,4099) '   min=',rmin
      rsum=rsum*FF1
      write(*,4098) '   Points in asym. unit= ',nasym
      write(*,4099) '   Scattering power sum= ', rsum
      print *,'Please wait ...'

C     *** reconstruct the whole-cell density-distribution DELC
      CALL RECNST(mpix,NPIX,masym,nasym,RDEN,IASYM,CELL,
     $   nsym,ncen,nsub,IROT,RTRS,RSUB,DELC)
C
      rsumold=rsum
      rsum=0
      do 40 i=0,NPIX(1)-1
      do 40 j=0,NPIX(2)-1
      do 40 k=0,NPIX(3)-1
	rsum=rsum+DELC(i,j,k)
40    continue
      rsum = rsum*FF1
      write(*,4099) ' Total scattering power=   ', rsum
      write(*,4099) ' Ratio to asym. unit sc.p.=', rsum/rsumold
C
C***  now select sections and make files for the SURFER graphics program ***
C
   50 WRITE(*,1000)
      WRITE(*,3040)
      READ(*,2000) mapt
      if (mapt.eq.0) then
        write(*,*) 'Finished.'
        STOP
      elseif (mapt.eq.1) then
        CALL MAP110(mapt,mpix,NPIX,nmx,nmy,vx,vy,theta,rad,
     $              CELL,DELC,D,lx,ly,dmin,dmax,axisX,axisY)
      elseif (mapt.eq.2.or.mapt.eq.3.or.mapt.eq.4) then
        CALL MAP100(mapt,mpix,NPIX,nmx,nmy,vx,vy,theta,rad,
     $              CELL,DELC,D,lx,ly,dmin,dmax,axisX,axisY)
      else
	GOTO 50
      endif
C     *** are you sure ? ***
      WRITE(*,3020)
      READ(*,2000) nyes
      IF (nyes.NE.0) GO TO 50
C
C     *** output of result ***
      CALL MECORES(mpix,rad,theta,vx,vy,dmin,dmax,
     $             mapt,lx,ly,axisx,axisy,D)
      GO TO 50
C
      END
