C
C--------------------------------------------------------------------
C     RECONSTRUCTS UNIT CELL DENSITY FROM ASYMMETRIC UNIT.
C--------------------------------------------------------------------
C
      SUBROUTINE RECNST(mpix,NPIX,masym,nasym,RDEN,IASYM,CELL,
     $   nsym,ncen,nsub,IROT,RTRS,RSUB,DELC)
C
C     DELC: reconstructed density in whole cell
C     CELL: cell parameters a,b,c,alpha,beta,gamma
C     NPIX: no. of pixels in cell-axes directions
C     RDEN: array with density in asymmetric unit
C     IASYM: corresponding ix,iy,iz coordinates of pixels in a.u.
C            ix=0...NPIX(1)-1, iy=0...NPIX(2)-1, iz=0...NPIX(3)-1
C     IROT,RTRS,RSUB: symmetry operators: rotation matrix,translation,
C                     and cell sub-translation for centered cells
C
      IMPLICIT  none
      integer*4 ia,masym,nasym
      integer mpix,ic,it,is,ncen,nsub,nsym
      integer NPIX(3),IASYM(3,masym),IROT(3,3,48)
      real*8 RDEN(masym),CELL(6)
      real*8 RTRS(3,48),RSUB(3,4),DELC(0:MPIX,0:MPIX,0:MPIX)
      integer kx,ky,kz,nx,ny,nz
      real*8 x,y,z,rSx,rSy,rSz,rx,ry,rz
C
      DO 10 ia=1,nasym
         kx=iasym(1,ia)
         ky=iasym(2,ia)
         kz=iasym(3,ia)
	 x=kx/DFLOAT(npix(1))
	 y=ky/DFLOAT(npix(2))
	 z=kz/DFLOAT(npix(3))
      DO 10 ic=1,ncen+1
      DO 10 it=1,nsub
         rSx=RSUB(1,it)
         rSy=RSUB(2,it)
         rSz=RSUB(3,it)
      DO 10 is=1,nsym
         rx=IROT(1,1,is)*x+IROT(1,2,is)*y+IROT(1,3,is)*z+RTRS(1,is)
         ry=IROT(2,1,is)*x+IROT(2,2,is)*y+IROT(2,3,is)*z+RTRS(2,is)
         rz=IROT(3,1,is)*x+IROT(3,2,is)*y+IROT(3,3,is)*z+RTRS(3,is)
         IF (IC.EQ.2) THEN
           nx=NINT( DMOD(2-rx+rSx,1d0)*NPIX(1) )
           ny=NINT( DMOD(2-ry+rSy,1d0)*NPIX(2) )
           nz=NINT( DMOD(2-rz+rSz,1d0)*NPIX(3) )
 	 ELSE
           nx=NINT( DMOD(2+rx+rSx,1d0)*NPIX(1) )
           ny=NINT( DMOD(2+ry+rSy,1d0)*NPIX(2) )
           nz=NINT( DMOD(2+rz+rSz,1d0)*NPIX(3) )
         END IF
         DELC(nx,ny,nz)=RDEN(ia)
   10 CONTINUE
C
      RETURN
      END
C
